FUND_TYPE = (
    '普通股票型基金',
    '被动指数型基金',
    '增强指数型基金',
    '偏股混合型基金',
    '平衡混合型基金',
    '偏债混合型基金',
    '灵活配置型基金',
    '中长期纯债型基金',
    '短期纯债型基金',
    '混合债券型一级基金',
    '混合债券型二级基金',
    '被动指数型债券基金',
    '增强指数型债券基金',
)